#include <stdio.h>
#include <X11/X.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>

#include "dat.h"
#include "fns.h"
#include "config.h"

#include "vector.h"

/* map is a VECTOR of workspaces
 * workspace is a VECTOR of clients
 */

static int current_workspace = 0;
static VECTOR *map;

void workspace_init( void ) {
    int i;
    map = vector_init();
    for( i=0; i<MAX_WORKSPACE; i++ ) {
        workspace_more();
    }
    set_workspace( 0 );
}

/* Add one more workspace to the window manager */
void workspace_more( void ) {
    VECTOR *workspace = vector_init();
    vector_push( map, workspace );
    set_workspace( vector_len( map ) - 1 );
}

/* Remove current workspace from the window manager */
void workspace_less( void ) {
    int wp;
    VECTOR *workspace = vector_at( map, current_workspace );
fprintf( stderr, "Current: %d\n", current_workspace );
fprintf( stderr, "Map size: %d\n", vector_len( map ) );
fprintf( stderr, "Workspace size: %d\n", vector_len( workspace ) );
fprintf( stderr, "--\n" );
    if( !workspace ) {
        fprintf( stderr, "workspace_delete: missing current workspace.\n" );
        return;
    }
    if( vector_len( map ) < 2 ) {
        fprintf( stderr, "workspace_delete: can't delete last workspace.\n" );
        return;
    }
    if( vector_len( workspace ) == 0 ) {
        vector_del_at( map, current_workspace );
        vector_destroy( workspace, NULL, NULL );
        wp = (current_workspace - 1);
        if( wp < 0 )
            wp = vector_len( map ) - 1;
        workspace_restore( wp );
        current_workspace = wp;
    } else {
        fprintf( stderr, "workspace_delete: can't do, workspace isn't empty.\n" );
    }
}

void workspace_delete( Client *c ) {
	int i;
	for( i=0; i<vector_len( map ); i++ ) {
        VECTOR *workspace = vector_at( map, i );
        vector_del( workspace, c );
    }
}

/* Return the workspace a client is mapped, or -1 if not mapped */
int workspace_mapped( Client *c ) {
	int i;
	for( i=0; i<vector_len( map ); i++ ) {
        VECTOR *workspace = vector_at( map, i );
        int pos = vector_find( workspace, c );
        if( pos != -1 ) {
            return( i );
        }
    }
    return( -1 );
}

/* Hide a client. */
void workspace_hide( Client *c ) {
	if( !c )
		return;
	XUnmapWindow( dpy, c->parent );
	XUnmapWindow( dpy, c->window );
	mysetstate( c, WithdrawnState );
	if( c == current )
		nofocus();
}

/* Unhide a client. */
void workspace_unhide( Client *c ) {
	if( !c )
		return;
	XMapWindow( dpy, c->window );
	XMapRaised( dpy, c->parent );
	mysetstate( c, NormalState );
	active( c );
	/* We do not call top here. It's unhide that calls top. */
	/* top( c ); */
}

void set_workspace_from_client( Client *c ) {
    int w = workspace_mapped( c );
    if( (w >= 0)&&( w != current_workspace) ) {
        set_workspace( w );
    }
}

/* Save the clients in the workspace, and hide them if requested. */
void workspace_save( int workspace_index, int hide ) {
	Client *c;
    VECTOR *workspace;
	for( c = clients; c; c = c->next ) {
		if( normal( c ) ) {
            workspace = vector_at( map, workspace_index );
            vector_push( workspace, c );
            if( hide ) {
                workspace_hide( c );
            }
        }
    }
}

/* Unhide the clients from the workspace. */
void workspace_restore( int workspace_index ) {
	Client *c;
    VECTOR *workspace;
    workspace = vector_at( map, workspace_index );

	for( c = vector_pop( workspace ); c != NULL; c = vector_pop( workspace ) ) {
		workspace_unhide( c );
	}
}

/* Change the workspace, hiding and unhiding clients as needed. */
void set_workspace( int new_workspace ) {
	int i;
fprintf( stderr, "set_workspace( %d )\n", new_workspace );
	if( ( new_workspace >= vector_len( map ) )
	    ||( new_workspace < 0 )
	    ||( new_workspace == current_workspace ) )
		return;

	/* Save old Workspace */
    workspace_save( current_workspace, 1 );

	current_workspace = new_workspace;

	/* Get New WorkSpace Clients */
    workspace_restore( current_workspace );
}

void prev_workspace( void ) {
	int wp = (current_workspace - 1);
	if( wp < 0 )
		wp = vector_len( map ) - 1;
	set_workspace( wp );
}

void next_workspace( void ) {
	int wp = (current_workspace + 1);
	if( wp >= vector_len( map ) )
		wp = 0;
	set_workspace( wp );
}
